/*
 * Java
 *
 * Copyright 2015-2021 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.kf;

import java.util.ArrayList;

import ej.kf.Feature;
import ej.kf.Kernel;
import ej.lang.Resource;
import ej.lang.ResourceManager;

/**
 * {@link KFResourceManager} is responsible for registering/unregistering created resource on Features.
 * These resources are closed before killing the Feature.
 */
public class KFResourceManager extends ResourceManager {

	@Override
	public void resourceCreated(Resource resource) {
		super.resourceCreated(resource);
		Feature f = Kernel.getContextOwnerFeature();
		if(f != null){
			ArrayList<Resource> resources = f.openResources;
			assert (resources != null);
			synchronized (resources) {
				resources.add(resource);
			}
		}
	}

	@Override
	public void resourceReclaimed(Resource resource) {
		super.resourceReclaimed(resource);
		Feature f = Kernel.getContextOwnerFeature();
		if(f != null){
			ArrayList<Resource> resources = f.openResources;
			assert (resources != null);
			synchronized (resources) {
				resources.remove(resource);
			}
		}
	}
}
