/*
 * Java
 *
 * Copyright 2013-2021 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.kf;

import ej.kf.Feature;
import ej.kf.Kernel;
import ej.kf.Module;
import ej.kf.Proxy;

/**
 * Default stub implementation (all is in Kernel mode)
 */
public class KernelNatives {

	public static boolean initialize(){
		return true;
	}

	public static int getMaxNbInstalledFeatures(){
		return 0;
	}

	public static long getInstalledFeatureDescriptor(int offset){
		throw new AssertionError();
	}

	public static ModuleData getData(long staticDescriptor){
		if(staticDescriptor == KernelNativesConstants.KERNEL_DESCRIPTOR){
			// same settings as in SOAR when no kernel description file
			char[] name = new char[] { 'K', 'E', 'R', 'N', 'E', 'L' };
			char[] version = new char[0];
			int nbFields = 6;
			char[][] ownerFields = new char[6][];
			for(int i=nbFields; --i>=0;){
				ownerFields[i] = new char[0];
			}
			PrincipalData owner = new PrincipalData(ownerFields);
			byte[] uid = new byte[0];
			return new ModuleData(name, version, owner, uid);
		}
		else {
			throw new RuntimeException(); // cannot happen
		}
	}

	public static long loadStaticFeatureByName(char[] name, int length){
		// not allowed
		return KernelNativesConstants.ERROR_LOADSTATIC_NOT_FOUND;
	}

	public static Class getEntryPointClass(Feature feature){
		throw new RuntimeException();// cannot happen
	}

	public static void setOwner(Object o, Feature feature){
		throw new RuntimeException();// cannot happen
	}

	/**
	 * Get the runtime descriptor of the owner of the given Object
	 * @return {@link #KERNEL_DESCRIPTOR} or a positive integer
	 */
	public static Module getOwner(Object o){
		return Kernel.getInstance(); // always Kernel
	}
	/**
	 * Get the runtime descriptor of the owner of the current context
	 * @return {@link #KERNEL_DESCRIPTOR} or a positive integer
	 */
	public static Module getContextOwner(){
		return Kernel.getInstance(); // always Kernel
	}

	public static void enter(){
		// NOP
	}

	public static void exit(){
		// NOP
	}

	public static void execClinitWrapper(Feature feature){
		throw new RuntimeException();// cannot happen
	}

	public static void linkToNative(Feature feature, long featureSectionDescriptor){

	}

	public static void unlinkToNative(Feature feature){

	}

	public static long loadFeatureByIndex(long featuresHeaderStart, int index){
		return 0;
	}

	public static boolean canLoadDynamicFeature(){
		return false;
	}

	public static boolean checkKernel(byte[] expectedKernelSignature, boolean expectedKernelEndianness) {
		return false;
	}

	/**
	 * Kill the given feature.
	 */
	public static void kill(Feature feature){

	}

	public static Object clone(Object from, Feature feature) throws CloneNotSupportedException{
		throw new CloneNotSupportedException();
	}

	public static Proxy bind(Object o, Class<?> targetInterface){
		throw new IllegalArgumentException();
	}

	public static Proxy newAnonymousProxy(Object o, Feature feature){
		throw new IllegalArgumentException();
	}

	public static boolean areEquivalentSharedInterfaces(Class<?> targetRef, Class<?> otherTargetRef){
		return false;
	}

	public static boolean isSharedInterface(Class<?> c){
		return false;
	}

	public static boolean isAPI(Class<?> c){
		return false;
	}

	public static Class<?> getImplementedSharedInterface(Class<?> c, Class<?> topInterface){
		return null;
	}

	public static Class<?> getTargetSharedInterface(Class<?> si, Class<?> topInterface, Feature targetFeature){
		return null;
	}

	public static void setStarted(Feature feature){
		// Nothing to do
	}

	public static int getState(Feature feature){
		return 1;
	}

	public static int getCriticality(Feature feature){
		return Feature.NORM_CRITICALITY;
	}

	public static void setCriticality(Feature feature, int criticatily){
		// Nothing to do
	}


	public static Feature getNextStoppedFeatureToUpdate(){
		throw new RuntimeException();// cannot happen
	}
}
