package com.is2t.natives.sp;

public abstract class NShieldedPlug {

	//same IDs as in iceTea (NShieldedPlug.iceTea) and in C (ShieldedPlug.h)
	//errors
	public static final int SUCCESS = 0;
	public static final int NO_BLOCK = -1;
	public static final int EMPTY_BLOCK = -2;
	public static final int INTERRUPTED = -3;
	public static final int TOO_MANY_WAITING = -4;
	public static final int INDEX_OUT_OF_BOUNDS = -5;

	//waiting state
	public static final int WAITING_FOR = -6;

	//data states
	public static final int DISABLE = 0;
	public static final int ENABLE = 1;
	public static final int PENDING = 2;

	//database
	public static native int getDatabasePtr(int ID);
	public static native int getSize(int databasePtr);
	public static native int getID(int databasePtr);
	public static native void getIDs(int databasePtr, int[] IDs);

	//block info
	public static native int exists(int databasePtr, int blockID);
	public static native int getLength(int databasePtr, int blockID);
	public static native int getMaxTasks(int databasePtr, int blockID);

	//read/write
	public static native int getDataState(int databasePtr, int blockID);
	public static native int reset(int databasePtr, int blockID);
	
	/**
	 * Read block length bytes in data from offset.<br>
	 * This method checks array bounds.<br>
	 * If fillAllTheArray is true, all the array from offset must be filled, if data.length-offset is higher than
	 * block length, INDEX_OUT_OF_BOUNDS is returned.
	 */
	public static native int read(int databasePtr, int blockID, byte[] data, int offset, boolean fillAllTheArray);
	

	/**
	 * Write block length bytes from data from offset.<br>
	 * This method checks array bounds.<br>
	 * If writeAllTheArray is true, all the array from offset must be written, if data.length-offset is higher than
	 * block length, INDEX_OUT_OF_BOUNDS is returned.
	 */
	public static native int write(int databasePtr, int blockID, byte[] data, int offset, boolean writeAllTheArray);

	//wait
	public static native int waitOne(int databasePtr, int blockID) throws InterruptedException;
	public static native int waitSeveral(int databasePtr, int[] blockIDs) throws InterruptedException;
	public static native int removeWaitFlag(int databasePtr, int blockID);
	public static native int removeWaitFlags(int databasePtr, int[] blockIDs, boolean[] writtenBlocksIDs);

}
