/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqCreateBlock extends SPInstanceRequest {
	//Request : <blockId(int)><length(int)><maxTasks(int)>
	//Reply : nothing
	//Throws : IllegalArgumentException, SecurityException
	//Return : void

	public int blockID;
	public int length;
	public int maxTasks;

	public ReqCreateBlock(ShieldedPlug sp, int blockID, int length, int maxTasks) {
		super(sp, REQ_CreateBlock);
		this.blockID = blockID;
		this.length = length;
		this.maxTasks = maxTasks;
	}

	public int computeDataLengthIntern() {
		return INT_SIZE * 3;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		dos.writeInt(blockID);
		dos.writeInt(length);
		dos.writeInt(maxTasks);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength == 0);
		return null;
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		if(SANITY_CHECK)
			spAssert(dataLength == INT_SIZE * 3);
		return new ReqCreateBlock(sp, dis.readInt(), dis.readInt(), dis.readInt());
	}

	public void execute() throws EmptyBlockException {
		sp.create(blockID, length, maxTasks);
	}

	public int getReplyDataLength() {
		return 0;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		//nothing to do
	}
}
