/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqGetIDs extends SPInstanceRequest {

	//Request : nothing
	//Reply : <ids(int)>*(dataLength/INT_SIZE)
	//Throws : nothing
	//Return : int[]

	int[] ids;

	public ReqGetIDs(ShieldedPlug sp) {
		super(sp, REQ_GetIDs);
	}

	public int computeDataLengthIntern() {
		return 0;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		//nothing to do
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		int nbIDs = dataLength/INT_SIZE;
		int[] ids = new int[nbIDs];
		//keep order
		for(int i=0 ; i<nbIDs ; i++)
			ids[i] = dis.readInt();
		return ids;
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		if(SANITY_CHECK)
			spAssert(dataLength == 0);
		return new ReqGetIDs(sp);
	}

	public void execute() throws EmptyBlockException {
		ids = sp.getIDs();
	}

	public int getReplyDataLength() {
		return ids.length*INT_SIZE;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		int[] ids = this.ids;
		int nbIDs = ids.length;
		for(int i=0 ; i<nbIDs ; i++)
			dos.writeInt(ids[i]);
	}

}
