/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqGetLength extends SPInstanceRequest{

	//Request : <blockId(int)>
	//Reply : <length(int)>
	//Throws : IllegalArgumentException
	//Return : Integer

	public int blockId;

	public int length;

	public ReqGetLength(ShieldedPlug sp, int blockId) {
		super(sp, REQ_GetLength);
		this.blockId = blockId;
	}
	public int computeDataLengthIntern() {
		return INT_SIZE;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		dos.writeInt(blockId);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength == INT_SIZE);

		return new Integer(dis.readInt());
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		if(SANITY_CHECK)
			spAssert(dataLength == INT_SIZE);
		return new ReqGetLength(sp, dis.readInt());
	}

	public void execute() throws EmptyBlockException {
		length = sp.getLength(blockId);
	}
	public int getReplyDataLength() {
		return INT_SIZE;
	}
	public void sendReply(DataOutputStream dos) throws IOException {
		dos.writeInt(length);
	}

}
