/*
 * Java
 *
 * Copyright 2011-2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import com.is2t.sp.remote.RequestExecutorPool;

import ej.sp.EmptyBlockException;

/**
 * Ask to interrupt an asynchronous request (waitFor)
 */
public class ReqInterruptRequest extends SPRequest{

	//Request : <requestId(int)>
	//Reply : nothing
	//Throws : nothing
	//Return : void

	public int requestID;

	public RequestExecutorPool requestExecutorPool;

	/** Build a Request that will be sent */
	public ReqInterruptRequest(int requestID){
		this(requestID, null);
	}

	/** Build a Request from received data */
	public ReqInterruptRequest(int requestID, RequestExecutorPool requestExecutorPool) {
		super(REQ_InterruptRequest);
		this.requestID = requestID;
		this.requestExecutorPool = requestExecutorPool;

	}
	public int computeDataLength() {
		return INT_SIZE;
	}

	public void sendRequest(DataOutputStream dos) throws IOException {
		dos.writeInt(requestID);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK) {
			spAssert(dataLength == 0);
		}

		return null;
	}

	public static SPRequest buildRequestFromStream(DataInputStream dis, int dataLength, RequestExecutorPool requestExecutorPool) throws IOException{
		if(SANITY_CHECK) {
			spAssert(dataLength == INT_SIZE);
		}
		return new ReqInterruptRequest(dis.readInt(), requestExecutorPool);
	}

	public void execute() throws EmptyBlockException {
		SPAsynchronousRequest asyncRequest = requestExecutorPool.getAsyncRequest(requestID);
		if(asyncRequest != null){
			Thread executor = asyncRequest.getRequestThread();
			if(executor != null){
				executor.interrupt();
			}
			asyncRequest.waitForEndOfExecution();
		}
	}

	public int getReplyDataLength() {
		return 0;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		//nothing to do
	}

}
