/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;


public class ReqIsBigEndian extends SPRequest {

	//Request : nothing
	//Reply : <endianess(byte)>
	//Throws : nothing
	//Return : DataReaderWriter

	private boolean isBigEndian;

	public ReqIsBigEndian() {
		super(REQ_IsBigEndian);
	}

	protected int computeDataLength() {
		return 0;
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength == BYTE_SIZE);

		isBigEndian = dis.readBoolean();

		return new Boolean(isBigEndian);
	}

	public void sendRequest(DataOutputStream dos) throws IOException {
		//nothing to do
	}

	public static SPRequest buildRequestFromStream(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength==0);

		return new ReqIsBigEndian();
	}

	public void execute() throws EmptyBlockException {
		int value = 1;

		isBigEndian = ((value & 0xFF) != 0);
	}

	public int getReplyDataLength() {
		return BYTE_SIZE;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		dos.writeBoolean(isBigEndian);
	}


}
