/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqIsDataAvailable extends SPInstanceRequest {

	//Request : <blockID(int)>
	//Reply : <available(boolean)>
	//Throws : IllegalArgumentException
	//Return : Boolean

	public int blockID;

	public boolean isAvailable;

	public ReqIsDataAvailable(ShieldedPlug sp, int blockID) {
		super(sp, REQ_IsDataAvailable);
		this.blockID = blockID;
	}

	public int computeDataLengthIntern() {
		return INT_SIZE;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		dos.writeInt(blockID);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength==BOOLEAN_SIZE);

		return new Boolean(dis.readBoolean());
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		if(SANITY_CHECK)
			spAssert(dataLength == INT_SIZE);
		return new ReqIsDataAvailable(sp, dis.readInt());
	}

	public void execute() throws EmptyBlockException {
		isAvailable = sp.isDataAvailable(blockID);
	}

	public int getReplyDataLength() {
		return BOOLEAN_SIZE;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		dos.writeBoolean(isAvailable);
	}

}
