/*
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqReadArray extends SPInstanceRequest{

	//Request : <blockID(int)> <length(int)>
	//Response : <data(byte*dataLength)>
	//Throws : IllegalArgumentException, EmptyBlockException, IndexOutOfBoundsException, ArrayIndexOutOfBoundsException
	//Return : void

	public int blockID;
	public byte[] data;
	public int offset;

	public ReqReadArray(ShieldedPlug sp, int blockID, byte[] data, int offset) {
		super(sp, REQ_ReadArray);
		this.blockID = blockID;
		this.data = data;
		this.offset = offset;
	}

	public int computeDataLengthIntern() {
		return 2*INT_SIZE;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		dos.writeInt(blockID);
		dos.writeInt(data.length);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		dis.readFully(data, offset, dataLength);
		return new Object();
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		if(SANITY_CHECK)
			spAssert(dataLength == 2*INT_SIZE);
		int blockID = dis.readInt();
		byte[] data = new byte[dis.readInt()];
		return new ReqReadArray(sp, blockID, data, 0);
	}

	public void execute() throws EmptyBlockException {
		sp.read(blockID, data, offset);
	}

	public int getReplyDataLength() {
		return data.length;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		dos.write(data, offset, data.length);
	}


}
