/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqReset extends SPInstanceRequest {

	//Request : <blockID(int)>
	//Reply : nothing
	//Throws : IllegalArgumentException
	//Return : void

	public int blockID;

	public ReqReset(ShieldedPlug sp, int blockID) {
		super(sp, REQ_Reset);
		this.blockID = blockID;
	}

	public int computeDataLengthIntern() {
		return INT_SIZE;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		dos.writeInt(blockID);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength==0);
		return null;
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		if(SANITY_CHECK)
			spAssert(dataLength == INT_SIZE);
		return new ReqReset(sp, dis.readInt());
	}

	public void execute() throws EmptyBlockException {
		sp.reset(blockID);
	}

	public int getReplyDataLength() {
		return 0;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		//nothing to do
	}

}
