/*
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.EmptyBlockException;
import ej.sp.ShieldedPlug;

public class ReqWriteArray extends SPInstanceRequest {

	//Request : <blockID(int)> <data(byte*length)>
	//Response : nothing
	//Throws : IllegalArgumentException, IndexOutOfBoundsException, ArrayIndexOutOfBoundsException
	//Return : void

	public int blockID;
	public byte[] data;
	public int offset;

	public ReqWriteArray(ShieldedPlug sp, int blockID, byte[] data, int offset) {
		super(sp, REQ_WriteArray);
		this.blockID = blockID;
		this.data = data;
		this.offset = offset;
	}

	public int computeDataLengthIntern() {
		return INT_SIZE + data.length;
	}

	public void sendRequestIntern(DataOutputStream dos) throws IOException {
		dos.writeInt(blockID);
		dos.write(data, offset, data.length);
	}

	protected Object computeReplyResult(DataInputStream dis, int dataLength) throws IOException {
		if(SANITY_CHECK)
			spAssert(dataLength==0);
		return null;
	}

	public static SPInstanceRequest buildRequestFromStream(ShieldedPlug sp, DataInputStream dis, int dataLength) throws IOException{
		int blockID = dis.readInt();
		byte[] data = new byte[dataLength - INT_SIZE];
		dis.readFully(data);
		return new ReqWriteArray(sp, blockID, data, 0);
	}

	public void execute() throws EmptyBlockException {
		sp.write(blockID, data, offset);
	}

	public int getReplyDataLength() {
		return 0;
	}

	public void sendReply(DataOutputStream dos) throws IOException {
		//nothing to do
	}

}
