/**
 * Java
 *
 * Copyright 2011-2015 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.sp.remote.request;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import ej.sp.ShieldedPlug;

/**
 *	Request that must be called on a ShieldedPlugInstance.
 *	Subclasses does not need to write databaseID when sending request.
 */
abstract public class SPInstanceRequest extends SPRequest{

	//Request : <databaseID(32)> <data>
	//Reply : <data>

	public ShieldedPlug sp;

	public SPInstanceRequest(ShieldedPlug sp, int commandID) {
		super(commandID);
		this.sp = sp;
	}

	protected int computeDataLength() {
		return INT_SIZE + computeDataLengthIntern();
	}

	public void sendRequest(DataOutputStream dos) throws IOException{
		dos.writeInt(sp.getID());
		sendRequestIntern(dos);
	}

	public static ShieldedPlug getShieldedPlug(DataInputStream dis) throws IOException{
		return ShieldedPlug.getDatabase(dis.readInt());
	}

	/**
	 * Returns request data length (without databaseID length).
	 */
	abstract public int computeDataLengthIntern();
	abstract public void sendRequestIntern(DataOutputStream dos) throws IOException;
}
