/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2014-2016, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.is2t.support.net.natives;

import com.is2t.errors.NetErrors;

public class StreamSocketChannelNatives {

	/**
	 * Reads {@code length} bytes from the socket associated with the file descriptor {@code fd}
	 * into the specified destination buffer {@code dst}.
	 * 
	 * @param fd the socket file descriptor
	 * @param kind the kind of the socket (0 for streaming, 1 if datagrams)
	 * @param dst the destination buffer into which the read data is stored
	 * @param offset the start offset in the buffer at which the data is written
	 * @param length the maximum number of data to read
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return the number of bytes read; -1 if there is no more data because the end of the stream has been reached or a negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native int readByteBufferNative(int fd, int kind, byte[] dst, int offset, int length, boolean retry);

	/**
	 * Writes {@code length} bytes from the specified buffer {@code buf} starting at the {@code offset} to the output stream
	 * of the socket associated with the file descriptor {@code fd}.
	 * 
	 * @param fd the socket file descriptor
	 * @param kind the kind of the socket (0 for streaming, 1 if datagrams)
	 * @param buf the buffer of data to write
	 * @param offset the offset in the data buffer
	 * @param length the number of data to write
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return the number of bytes written or a negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native int writeByteBufferNative(int fd, int kind, byte[] buf, int offset, int length, boolean retry);

	/**
	 * Accepts an incoming connection for the listening socket associated with the file descriptor {@code fd}
	 * and returns the file descriptor of the accepted socket.
	 * 
	 * @param fd the socket file descriptor that listening for incoming connection
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return the file descriptor of the accepted socket or a negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native int accept(int fd, boolean retry);

	/**
	 * Returns an estimate of the number of bytes that can be read from the socket associated with the file descriptor {@code fd}
	 * without blocking by the next invocation of a read operation.
	 * 
	 * @param fd the socket file descriptor
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return an estimate of the number of bytes that can be read from the socket associated with the file descriptor {@code fd}
	 * without blocking by the next invocation of a read operation or a negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native int available(int fd, boolean retry);


}
