/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2018, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.is2t.support.net.ssl;

import ej.error.ErrorMessages;

public class SSLErrorMessages implements ErrorMessages, SSLErrorConstants {

	private static final String CategorySSL = "SSL-2.1";

	private static SSLErrorMessages instance = new SSLErrorMessages();

	public static SSLErrorMessages getInstance(){
		return instance;
	}

	@Override
	public String category() {
		return CategorySSL;
	}

	@Override
	public String messageAt(int err) {

		if(err >= J_SSL_NO_ERROR){
			return "";
		}
		switch (err) {
			case J_CONNECTION_RESET:
				return "Connection reset";
			case J_BLOCKING_QUEUE_LIMIT_REACHED:
				return "Internal dispatch blocking queue limit reached";
			case J_CERT_PARSE_ERROR:
				return "Unable to parse certificate";
			case J_BIG_CERT_OR_KEY_SIZE_ERROR:
				return "Certificate or key size is bigger than the immortal buffer used to process certificates and keys. Increase the maximum certificate and key size";
			case J_NO_TRUSTED_CERT:
				return "No trusted certificate found";
			case J_NOT_CA_CERT:
				return "basic constraints check failed: intermediate certificate is not a CA certificate";
			case J_NAME_CHAINING_ERROR:
				return "subject/issuer name chaining error";
			case J_BIG_KEY_PASSWORD_SIZE_ERROR:
				return "Key password size bigger than the immortal buffer reserved for the key passwords";
			case J_RSA_WRONG_TYPE_ERROR:
				return "RSA wrong block type for RSA function";
			case J_RSA_BUFFER_ERROR:
				return "RSA buffer error: output too small or input too big";
			case J_BUFFER_ERROR:
				return "Buffer error: output too small or input too big";
			case J_ALGO_ID_ERROR:
				return "Setting Certificate AlogID error";
			case J_PUBLIC_KEY_ERROR:
				return "Setting Certificate Public Key error";
			case J_DATE_ERROR:
				return "Setting Certificate Date validity error";
			case J_SUBJECT_ERROR:
				return "Setting Certificate Subject name error";
			case J_ISSUER_ERROR:
				return "Setting Cert Issuer name error";
			case J_CA_TRUE_ERROR:
				return "Setting basic constraint CA true error";
			case J_EXTENSIONS_ERROR:
				return "Setting extensions error";
			case J_ASN_VERSION_ERROR:
				return "ASN version error, invalid number";
			case J_ASN_GETINT_ERROR:
				return "ASN get big int error, invalid data";
			case J_ASN_RSA_KEY_ERROR:
				return "ASN key init error, invalid input";
			case J_ASN_OBJECT_ID_ERROR:
				return "ASN object id error, invalid id";
			case J_ASN_TAG_NULL_ERROR:
				return "ASN tag error, not null";
			case J_ASN_EXPECT_0_ERROR:
				return "ASN expect error, not zero";
			case J_ASN_BITSTR_ERROR:
				return "ASN bit string error, wrong id";
			case J_ASN_UNKNOWN_OID_ERROR:
				return "ASN oid error, unknown sum id";
			case J_ASN_DATE_SZ_ERROR:
				return "ASN date error, bad size";
			case J_ASN_BEFORE_DATE_ERROR:
				return "ASN date error, current date before";
			case J_ASN_AFTER_DATE_ERROR:
				return "ASN date error, current date after";
			case J_ASN_SIG_OID_ERROR:
				return "ASN signature error, mismatched oid";
			case J_ASN_TIME_ERROR:
				return "ASN time error, unkown time type";
			case J_ASN_INPUT_ERROR:
				return "ASN input error, not enough data";
			case J_ASN_SIG_CONFIRM_ERROR:
				return "ASN signature error, confirm failure";
			case J_ASN_SIG_HASH_ERROR:
				return "ASN signature error, unsupported hash type";
			case J_ASN_SIG_KEY_ERROR:
				return "ASN signature error, unsupported key type";
			case J_ASN_DH_KEY_ERROR:
				return "ASN key init error, invalid input";
			case J_ASN_NTRU_KEY_ERROR:
				return "ASN NTRU key decode error, invalid input";
			case J_ASN_CRIT_EXT_ERROR:
				return "X.509 Critical extension ignored";
			case J_ECC_BAD_ARG_ERROR:
				return "ECC input argument wrong type, invalid input";
			case J_ASN_ECC_KEY_ERROR:
				return "ECC ASN1 bad key data, invalid input";
			case J_ECC_CURVE_OID_ERROR:
				return "ECC curve sum OID unsupported, invalid input";
			case J_BAD_FUNC_ARG:
				return "Bad function argument";
			case J_NOT_COMPILED_IN:
				return "Feature not compiled in";
			case J_UNICODE_SIZE_ERROR:
				return "Unicode password too big";
			case J_NO_PASSWORD:
				return "No password provided by user";
			case J_ALT_NAME_ERROR:
				return "Alt Name problem, too big";
			case J_AES_GCM_AUTH_ERROR:
				return "AES-GCM Authentication check fail";
			case J_AES_CCM_AUTH_ERROR:
				return "AES-CCM Authentication check fail";
			case J_ASYNC_INIT_ERROR:
				return "Async Init error";
			case J_BAD_ALIGN_ERROR:
				return "Bad alignment error, no alloc help";
			case J_ASN_NO_SIGNER_ERROR:
				return "ASN no signer error to confirm failure";
			case J_ASN_CRL_CONFIRM_ERROR:
				return "ASN CRL signature error, confirm failure";
			case J_ASN_CRL_NO_SIGNER_ERROR:
				return "ASN CRL no signer error to confirm failure";
			case J_ASN_OCSP_CONFIRM_ERROR:
				return "ASN OCSP signature error, confirm failure";
			case J_BAD_STATE_ERROR:
				return "Bad state operation";
			case J_BAD_PADDING_ERROR:
				return "Bad padding, message wrong length";
			case J_REQ_ATTRIBUTE_ERROR:
				return "Setting cert request attributes error";
			case J_PKCS7_OID_ERROR:
				return "PKCS#7 error: mismatched OID value";
			case J_PKCS7_RECIP_ERROR:
				return "PKCS#7 error: no matching recipient found";
			case J_FIPS_NOT_ALLOWED_ERROR:
				return "FIPS mode not allowed error";
			case J_ASN_NAME_INVALID_ERROR:
				return "Name Constraint error";
			case J_RNG_FAILURE_ERROR:
				return "Random Number Generator failed";
			case J_HMAC_MIN_KEYLEN_ERROR:
				return "FIPS Mode HMAC Minimum Key Length error";
			case J_RSA_PAD_ERROR:
				return "Rsa Padding error";
			case J_LENGTH_ONLY_ERROR:
				return "Output length only set, not for other use error";
			case J_IN_CORE_FIPS_ERROR:
				return "In Core Integrity check FIPS error";
			case J_AES_KAT_FIPS_ERROR:
				return "AES Known Answer Test check FIPS error";
			case J_DES3_KAT_FIPS_ERROR:
				return "DES3 Known Answer Test check FIPS error";
			case J_HMAC_KAT_FIPS_ERROR:
				return "HMAC Known Answer Test check FIPS error";
			case J_RSA_KAT_FIPS_ERROR:
				return "RSA Known Answer Test check FIPS error";
			case J_DRBG_KAT_FIPS_ERROR:
				return "DRBG Known Answer Test check FIPS error";
			case J_DRBG_CONT_FIPS_ERROR:
				return "DRBG Continuous Test FIPS error";
			case J_AESGCM_KAT_FIPS_ERROR:
				return "AESGCM Known Answer Test check FIPS error";
			case J_INPUT_CASE_ERROR:
				return " ";
			case J_PREFIX_ERROR:
				return "bad index to key rounds";
			case J_MEMORY_ERROR:
				return "not enough memory";
			case J_VERIFY_FINISHED_ERROR:
				return "verify problem on finished";
			case J_VERIFY_MAC_ERROR:
				return "verify mac problem";
			case J_HEADER_PARSE_ERROR:
				return "parse error on header";
			case J_UNKNOWN_HANDSHAKE_TYPE:
				return "weird handshake type";
			case J_SOCKET_ERROR:
				return "error state on socket";
			case J_SOCKET_NODATA:
				return "expected data, not there";
			case J_INCOMPLETE_DATA:
				return "don't have enough data to complete task";
			case J_UNKNOWN_RECORD_TYPE:
				return "unknown type in record header";
			case J_DECRYPT_ERROR:
				return "error during decryption";
			case J_FATAL_ERROR:
				return "fatal alert";
			case J_ENCRYPT_ERROR:
				return "error during encryption";
			case J_NO_PEER_KEY:
				return "need peer's key";
			case J_NO_PRIVATE_KEY:
				return "need the private key";
			case J_RSA_PRIVATE_ERROR:
				return "error during rsa priv op";
			case J_NO_DH_PARAMS:
				return "server missing DH params";
			case J_BUILD_MSG_ERROR:
				return "build message failure";
			case J_BAD_HELLO:
				return "client hello malformed";
			case J_DOMAIN_NAME_MISMATCH:
				return "peer subject name mismatch";
			case J_WANT_READ:
				return "non-blocking socket wants data to be read";
			case J_NOT_READY_ERROR:
				return "handshake layer not ready yet, complete first";
			case J_PMS_VERSION_ERROR:
				return "premaster secret version mismatch error";
			case J_VERSION_ERROR:
				return "record layer version error";
			case J_WANT_WRITE:
				return "non-blocking socket write buffer full";
			case J_MALFORMED_BUFFER:
				return "malformed buffer input error";
			case J_VERIFY_CERT_ERROR:
				return "verify problem on certificate";
			case J_VERIFY_SIGN_ERROR:
				return "verify problem based on signature";
			case J_CLIENT_ID_ERROR:
				return "psk client identity error";
			case J_SERVER_HINT_ERROR:
				return "psk server hint error";
			case J_PSK_KEY_ERROR:
				return "psk key callback error";

			case J_LENGTH_ERROR:
				return "record layer length error";
			case J_PEER_KEY_ERROR:
				return "cant decode peer key";
			case J_ZERO_RETURN:
				return "receive close notify alert";
			case J_SIDE_ERROR:
				return "wrong client/server type";
			case J_NO_PEER_CERT:
				return "peer didn't send cert";
			case J_NTRU_KEY_ERROR:
				return "NTRU key error";
			case J_NTRU_DRBG_ERROR:
				return "NTRU drbg error";
			case J_NTRU_ENCRYPT_ERROR:
				return "NTRU drbg error";
			case J_NTRU_DECRYPT_ERROR:
				return "NTRU decrypt error";
			case J_ECC_CURVETYPE_ERROR:
				return "Bad ECC Curve Type or unsupported";
			case J_ECC_CURVE_ERROR:
				return "Bad ECC Curve or unsupported";
			case J_ECC_PEERKEY_ERROR:
				return "Bad ECC Peer Key";
			case J_ECC_MAKEKEY_ERROR:
				return "ECC Make Key failure";
			case J_ECC_EXPORT_ERROR:
				return "ECC Export Key failure";
			case J_ECC_SHARED_ERROR:
				return "ECC DHE shared failure";
			case J_NOT_CA_ERROR:
				return "Not a CA by basic constraint error";
			case J_BAD_CERT_MANAGER_ERROR:
				return "Bad Cert Manager error";
			case J_OCSP_CERT_REVOKED:
				return "OCSP Cert revoked";
			case J_CRL_CERT_REVOKED:
				return "CRL Cert revoked";
			case J_CRL_MISSING:
				return "CRL missing, not loaded";

			case J_OCSP_NEED_URL:
				return "OCSP need URL";
			case J_OCSP_CERT_UNKNOWN:
				return "OCSP Cert unknown";
			case J_OCSP_LOOKUP_FAIL:
				return "OCSP Responder lookup fail";
			case J_MAX_CHAIN_ERROR:
				return "Maximum Chain Depth Exceeded";

			case J_SUITES_ERROR:
				return "Suites Pointer Error";
			case J_SSL_NO_PEM_HEADER:
				return "No PEM Header Error";
			case J_OUT_OF_ORDER_ERROR:
				return "Out of order message, fatal";
			case J_BAD_KEA_TYPE_ERROR:
				return "Bad KEA type found";
			case J_SANITY_CIPHER_ERROR:
				return "Sanity check on ciphertext failed";
			case J_RECV_OVERFLOW_ERROR:
				return "Receive callback returned more than requested";

			case J_NO_PEER_VERIFY:
				return "Need peer certificate verify Error";

			case J_UNKNOWN_HOST_NAME_ERROR:
				return "Unrecognized host name Error";
			case J_UNKNOWN_MAX_FRAG_LEN_ERROR:
				return " ";
			case J_KEYUSE_SIGNATURE_ERROR:
				return "Key Use digitalSignature not set Error";
			case J_KEYUSE_ENCIPHER_ERROR:
				return "Key Use keyEncipherment not set Error";
			case J_EXTKEYUSE_AUTH_ERROR:
				return "Ext Key Use server/client auth not set Error";
			case J_SEND_OOB_READ_ERROR:
				return "Send Callback Out of Bounds Read Error";
			case J_SECURE_RENEGOTIATION_ERROR:
				return "Invalid Renegotiation Error";
			case J_SCR_DIFFERENT_CERT_ERROR:
				return "Peer sent different cert during SCR";
			case J_NO_CHANGE_CIPHER_ERROR:
				return "Finished received from peer before Change Cipher Error";
			case J_SANITY_MSG_ERROR:
				return "Sanity Check on message order Error";
			case J_DUPLICATE_MSG_ERROR:
				return "Duplicate HandShake message Error";

			case J_UNSUPPORTED_SUITE:
				return "unsupported cipher suite";
			case J_MATCH_SUITE_ERROR:
				return "can't match cipher suite";
			case J_BAD_CERTTYPE:
				return "Bad certificate type";
			case J_BAD_ENCODED_CERT_FORMAT:
				return "Unsupported certificate encoded format";

			case J_OPEN_RAN_ERROR:
				return "opening random device error";
			case J_READ_RAN_ERROR:
				return "reading random device error";
			case J_WINCRYPT_ERROR:
				return "windows crypt init error";
			case J_CRYPTGEN_ERROR:
				return "windows crypt generation error";
			case J_RAN_BLOCK_ERROR:
				return "random device read would block error";
			case J_UNKNOWN_ERROR:
			default:
				return "Unknown error ("+err+")";
		}
	}


}
