/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2017, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.is2t.support.net.ssl;

import java.io.IOException;
import java.io.InputStream;
import java.net.SocketException;

class SSLSocketInputStream extends InputStream{

	private final SSLSocketImpl socket;

	public SSLSocketInputStream(SSLSocketImpl socket) {
		this.socket = socket;
	}

	@Override
	public int read(byte[] b, int off, int length) throws IOException {

		//bounds check
		if (length <= 0 || off < 0 || off + length > b.length) {
			if (length == 0) {
				return 0;
			}
			throw new ArrayIndexOutOfBoundsException();
		}

		//do the handshake if it is not already done
		socket.doHandShake(false);

		int n = SSLSocketSupport.read(socket, b, off, length);
		if(n > 0){
			return n;
		}

		if(socket.isClosed()){
			throw new SocketException("Socket closed");
		}

		return -1;
	}

	@Override
	public int read() throws IOException {
		byte temp[] = new byte[1];
		int n = read(temp, 0, 1);
		if (n <= 0) {
			return -1;
		}
		return temp[0] & 0xff;
	}

	@Override
	public int available() throws IOException {
		if(socket.isClosed()){
			throw new IOException("Stream closed.");
		}
		if (!socket.isCreated()) {
			return 0; // the underlying ssl socket is not yet created
		}
		return SSLSocketSupport.available(socket);
	}

	@Override
	public void close() throws IOException {
		socket.close();
	}

}
