/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.is2t.support.net.ssl;

import java.io.IOException;
import java.io.OutputStream;
import java.net.SocketException;

class SSLSocketOuputStream extends OutputStream{

	private SSLSocketImpl socket;
	 
	public SSLSocketOuputStream(SSLSocketImpl socket) {
		this.socket = socket;
	}

	@Override
	public void write(byte[] b, int off, int length) throws IOException {
		
		//bounds check
        if (length <= 0 || off < 0 || off + length > b.length) {
            if (length == 0) {
                return;
            }
            throw new ArrayIndexOutOfBoundsException();
        }
        
		//do the handshake if it is not already done
		socket.doHandShake(false);
		
        try{
        	SSLSocketSupport.write(socket, b, off, length);
        }catch(IOException e){
        	if(socket.isClosed()){
            	throw new SocketException("socket closed");
            }
        	throw e;
        }
	}
	
	@Override
	public void write(int b) throws IOException {
		byte temp[] = new byte[1];
        temp[0] = (byte)b;
        write(temp, 0, 1);
	}
	
	@Override
	public void close() throws IOException {
		socket.close();
	}
	

}
