/*
 * Copyright (C) 2017, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.is2t.support.security;

import java.security.KeyFactorySpi;
import java.security.MessageDigestSpi;
import java.security.SecureRandomSpi;
import java.security.SignatureSpi;

import javax.crypto.CipherSpi;
import javax.crypto.MacSpi;

/**
 *
 */
public interface SecurityProvider {

	/**
	 * Returns a {@link CipherSpi} for the given transformation or null if no
	 * implementation is available.
	 */
	public CipherSpi getCipherSpi(String transformation);

	/**
	 * Returns a {@link SignatureSpi} for the given algorithm or null if no
	 * implementation is available.
	 */
	public SignatureSpi getSignatureSpi(String algorithm);

	/**
	 * Returns a {@link MessageDigestSpi} for the given algorithm or null if no
	 * implementation is available.
	 */
	public MessageDigestSpi getMessageDigestSpi(String algorithm);

	/**
	 * Returns a {@link MacSpi} for the given algorithm or null if no
	 * implementation is available.
	 */
	public MacSpi getMacSpi(String algorithm);

	/**
	 * Returns a {@link SecureRandomSpi} or null if no implementation is available.
	 */
	public SecureRandomSpi getSecureRandomSpi();

	/**
	 * Returns a {@link KeyFactorySpi} for the given algorithm or null if no
	 * implementation is available.
	 */
	public KeyFactorySpi getKeyFactorySpi(String algorithm);

}
