/*
 * Java
 *
 * Copyright 2015-2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.hal.gpio;

public class GPIO {

	public enum Mode {

		DIGITAL_INPUT,

		DIGITAL_INPUT_PULLUP,

		DIGITAL_OUTPUT,

		ANALOG_INPUT,

		ANALOG_OUTPUT;
	}

	/**
	 * Returned native value
	 */
	private static final int OK = 0;

	/**
	 * Returned native value
	 */
	private static final int NOK = -1;

	/**
	 * Returned native value
	 */
	private static final int DIGITAL_LOW = 0;

	/**
	 * Returned native value
	 */
	private static final int DIGITAL_HIGH = 1;

	public static void setMode(int port, int pin, Mode mode) {
		checkNativeReturnedValue(GPIONatives.setPinMode(port, pin, mode.ordinal()));
	}

	public static boolean getDigitalValue(int port, int pin) {
		return checkNativeReturnedValue(GPIONatives.getDigitalPinValue(port, pin)) == DIGITAL_HIGH;

	}

	public static void setDigitalValue(int port, int pin, boolean value) {
		checkNativeReturnedValue(GPIONatives.setDigitalPinValue(port, pin, value ? DIGITAL_HIGH : DIGITAL_LOW));
	}

	public static int getAnalogValue(int port, int pin) {
		return checkNativeReturnedValue(GPIONatives.getAnalogPinValue(port, pin));
	}

	public static void setAnalogValue(int port, int pin, int percentage) {
		if (percentage < 0 | percentage > 100) {
			throw new IllegalArgumentException();
		}
		checkNativeReturnedValue(GPIONatives.setAnalogPinValue(port, pin, percentage));
	}

	/*
	 * Private methods
	 */

	/**
	 * Throw an exception or not according the native returned value.
	 *
	 * @param ret
	 *            the native returned value.
	 * @return the native returned value when no error.
	 */
	private static int checkNativeReturnedValue(int ret) {
		if (ret == NOK) {
			throw new IllegalArgumentException();
		}
		return ret;
	}


}
