/*
 * Java
 *
 * Copyright 2013-2021 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.kf;

import ej.annotation.Nullable;

/**
 * This exception is thrown when a Feature data being loaded has an invalid format.
 * @see Kernel#load(java.io.InputStream)
 */
@SuppressWarnings("serial")
public class InvalidFormatException extends Exception{

	public static final int NO_ELIGIBLE_LOADER 		= 1;
	public static final int LOADER_FORMAT_ERROR 	= 2;
	public static final int FIRST_SPECIFIC_ERROR 	= 50; /* Keep in sync with the KFDynamicLoaderNativesPool in the vm */

	private final int kind;

	public InvalidFormatException(){
		// API constructor
		kind = 0;
	}

	public InvalidFormatException(int kind){
		this(kind, null);
	}

	public InvalidFormatException(int kind, @Nullable String message) {
		super(message);
		this.kind = kind;
	}

	@Override
	@Nullable
	public String getMessage(){
		String message = super.getMessage();
		return "code="+kind+(message==null?"":":"+message);
	}
}
