/*
 * Java
 *
 * Copyright 2016-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.microui.display;

import ej.kf.DeadFeatureException;
import ej.microui.MicroUIFactory;

/**
 * Special event for switching the display related by the {@link DisplayPump}.
 */
public class KFSetCurrentDisplayRunnable extends WaitForEventRunnable {

	private final KFDisplayPump displayPump;
	private final Display newDisplay;

	/**
	 * @param expectedDisplayOwner
	 *            when set, the display is switched if and only if it is the owner
	 *            of the DisplayPump at the time this event is processed (otherwise,
	 *            the Display is let as is)
	 */
	public KFSetCurrentDisplayRunnable(KFDisplayPump displayPump, Display newDisplay) {
		this.displayPump = displayPump;
		this.newDisplay = newDisplay;
	}

	@Override
	public void run() {
		KFDisplayPump pump = this.displayPump;
		Display previousDisplay = pump.display;

		// hide previous Displayable
		try {
			previousDisplay.replaceDisplayable(null);
		} catch (DeadFeatureException e) {
			// previous display being killed - skip
		}

		pump.setCurrentDisplay(this.newDisplay);

		MicroUIFactory.getInstance().resetEventHandlers(this.newDisplay);
		super.run();
	}

	@Override
	protected void notifyWaitingThread() {
		notifyAll(); // may have multiple waiting threads on the same event
	}

}
