
/*
 *	Java
 *	
 *	Copyright 2011-2015 IS2T. All rights reserved.
 *	IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package ej.sp;

/**
 * The <code>SPReader</code> interface provides a method for reconstructing objects from a block.
 */
public interface SPReader {

	/**
	 * Reads and returns an object from a block of the given {@link ShieldedPlug}.
	 * The class that implements this interface defines how the object is "read".
	 *
	 * @param sp
	 * 		the {@link ShieldedPlug} from which data is read
	 * @param blockID
	 * 		ID of the block
	 * @return
	 * 		the object read from the {@link ShieldedPlug}
	 * @throws IllegalArgumentException
	 * 		if no block is defined with the given ID
	 * @throws IndexOutOfBoundsException
	 * 		if block length is lower than the size needed for object deserialization
	 * @throws EmptyBlockException
	 * 		if no data is available in the block
	 */
	public Object readObject(ShieldedPlug sp, int blockID) throws EmptyBlockException;

}
