/*
 * Copyright (c) 1997, 2013, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2017, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */


package javax.net.ssl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketImpl;
import java.net.UnknownHostException;

public abstract class SSLSocket extends Socket
{
	protected SSLSocket()
	{ super(); }


	protected SSLSocket(SocketImpl impl) throws IOException {
		super(impl);
	}

	protected SSLSocket(String host, int port)
			throws IOException, UnknownHostException
	{ super(host, port); }

	protected SSLSocket(InetAddress address, int port)
			throws IOException
	{ super(address, port); }

	protected SSLSocket(String host, int port,
			InetAddress clientAddress, int clientPort)
					throws IOException, UnknownHostException
	{ super(host, port, clientAddress, clientPort); }


	protected SSLSocket(InetAddress address, int port,
			InetAddress clientAddress, int clientPort)
					throws IOException
	{ super(address, port, clientAddress, clientPort); }


	public abstract void startHandshake() throws IOException;

	public abstract void setNeedClientAuth(boolean need);

	public abstract boolean getNeedClientAuth();

	public abstract void setUseClientMode(boolean mode);

	public abstract boolean getUseClientMode();
}
